﻿Option Infer On
Option Explicit On
Option Strict On

Imports System
Imports System.Linq
Imports System.Collections.Generic
Imports KadGen.SLTestApp.Common
Imports System.ComponentModel.Composition
'Imports System.ComponentModel.DataAnnotations

Public MustInherit Class BizServiceAsyncBase(Of T)
   Implements IBizServiceAsync(Of T)

   <Import()>
   Public _creator As ExportFactory(Of T)

   Public Event RetrieveListCompleted(ByVal sender As Object, ByVal e As RetrieveListCompetedEventArgs(Of T)) Implements IBizServiceAsync(Of T).RetrieveListCompletedEvent

   'Public MustOverride ReadOnly Property CanReadProperty(ByVal propertyName As String) As Boolean Implements IBizService.CanReadProperty
   ' Public MustOverride Sub BeginRetrieveList(ByVal filter As IFilter) Implements IBizServiceAsync(Of T).BeginRetrieveList

   Private _searchProperties As IEnumerable(Of String)
   Protected Sub New(ByVal searchProperties As IEnumerable(Of String))
      _searchProperties = searchProperties
   End Sub

   Public Function CreateNew() As T Implements IBizServiceAsync(Of T).CreateNew
      ' HACK: There is a bit of messiness here between the InvoiceLineItem and IInvoiceLineItemStandard. It's retrieved here as a standard, but passed back as a plain adn then cast. But this allows future narrower values to be used
      Dim newItem = _creator.CreateExport.Value
      Return newItem
   End Function

   Public ReadOnly Property IsSearchProperty(ByVal propertyName As String) As Boolean Implements IBizService.IsSearchProperty
      Get
         Return _searchProperties.Contains(propertyName)
      End Get
   End Property

   Protected Overridable Sub OnRetrieveListCompleted(ByVal e As RetrieveListCompetedEventArgs(Of T))
      RaiseEvent RetrieveListCompleted(Me, e)
   End Sub

   ' Attempt at moving more code to base class
   <Import()>
   Public _retriever As ITargetTypePartRetriever
   Public WithEvents _wcfService As IWcfWrapper(Of T)

   Private ReadOnly Property WcfService() As IWcfWrapper(Of T)
      Get
         If _wcfService Is Nothing Then
            _wcfService = CType(_retriever.GetItem(Of IWcfWrapper)(GetType(T)), IWcfWrapper(Of T))
         End If
         Return _wcfService
      End Get
   End Property

   Public Overridable ReadOnly Property CanReadProperty(ByVal propertyName As String) As Boolean Implements IBizService.CanReadProperty
      Get
         If propertyName = "MoreInfo" Then
            Return False
         End If
         If propertyName = "InstanceState" Then
            Return False
         End If
         Return True
      End Get
   End Property

   Public Sub BeginRetrieveList(ByVal filter As IFilter) Implements IBizServiceAsync(Of T).BeginRetrieveList
      WcfService.BeginRetrieveList(filter)
   End Sub

   Private Sub _wcfService_RetrieveListCompletedEvent(ByVal sender As Object, _
                                                     ByVal e As RetrieveListCompetedEventArgs(Of T)) _
                                                     Handles _wcfService.RetrieveListCompletedEvent
      OnRetrieveListCompleted(e)
   End Sub

End Class
