﻿Option Strict On
Option Explicit On
Option Infer On

Imports System
Imports System.Linq
Imports System.Collections
Imports System.Windows.Controls
Imports System.ComponentModel.Composition
Imports System.Windows
Imports KadGen.SLTestApp.Common

<Export(GetType(IChildDataGrid))> _
<PartCreationPolicy(CreationPolicy.NonShared)> _
Partial Public Class ChildDataGrid
   Inherits UserControl
   Implements IChildDataGrid

   <Import()> _
   Public _retriever As ITargetTypePartRetriever

   Public Sub New()
      InitializeComponent()
   End Sub

   Public Sub Initialize( _
            ByVal propertyName As String, _
            ByVal propertyType As System.Type, _
            ByVal parenttype As System.Type) _
            Implements IChildDataGrid.Initialize
      _propertyName = propertyName
      _propertyType = propertyType
      _parentType = parenttype
      Dim binding = New Data.Binding(_propertyName)
      binding.Mode = Data.BindingMode.TwoWay
      dataGrid.SetBinding(dataGrid.ItemsSourceProperty, binding)
   End Sub

   Private _propertyName As String
   Public ReadOnly Property PropertyName() As String Implements IChildDataGrid.PropertyName
      Get
         Return _propertyName
      End Get
   End Property

   Private _propertyType As Type
   Public ReadOnly Property PropertyType() As Type Implements IChildDataGrid.PropertyType
      Get
         Return _propertyType
      End Get
   End Property

   Private _parentType As Type
   Public ReadOnly Property ParentType() As Type Implements IChildDataGrid.ParentType
      Get
         Return _parentType
      End Get
   End Property

   Private Sub SetChildGridColumns()
      If dataGrid.Columns.Count > 0 Then
         ' assume columns have already been set
         Return
      End If
      Dim vmType = GetInnerType()
      Dim q = From p In vmType.GetProperties _
              Where ViewModelMany.CanReadProperty(p.Name)
      For Each prop In q
         Dim column As DataGridColumn
         If prop.PropertyType Is GetType(Boolean) Then
            Dim x = New DataGridCheckBoxColumn()
            x.Binding = New Data.Binding(prop.Name)
            column = x
         Else
            Dim x = New DataGridTextColumn
            x.Binding = New Data.Binding(prop.Name)
            column = x
         End If
         column.Header = prop.Name
         dataGrid.Columns.Add(column)
      Next
   End Sub

   Private Function GetInnerType() As Type
      Dim ret = PropertyType.GetGenericArguments(0)
      If ret IsNot Nothing Then
         Dim temp = _retriever.GetItem(Of IViewModel)(ret)
         Return temp.GetType
      End If
      Throw New InvalidOperationException
   End Function

   Private ReadOnly Property CurrentViewModel() As IViewModel
      Get
         If dataGrid.SelectedItem IsNot Nothing Then
            Return CType(dataGrid.SelectedItem, IViewModel)
         End If
         Return Nothing
      End Get
   End Property

   Private ReadOnly Property ViewModelMany() As IViewModelMany
      Get
         Return CType(dataGrid.ItemsSource, IViewModelMany)
      End Get
   End Property

   Public Sub DeleteCurrent() Implements IChildDataGrid.DeleteCurrent
      Dim vm = CurrentViewModel
      If vm IsNot Nothing Then
         vm.Delete()
      End If
   End Sub

   Public Sub Insert() Implements IChildDataGrid.Insert
      ViewModelMany.CreateNew()
   End Sub

   Private Sub dataGrid_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles dataGrid.Loaded
      SetChildGridColumns()
   End Sub

End Class

