﻿Option Infer On
Option Explicit On
Option Strict On

Imports System
Imports System.Linq
Imports System.Collections.Generic
Imports System.ComponentModel.Composition
Imports KadGen.SLTestApp.Common

<PartRetriever()> _
Public Class PartRetriever

   Implements IPartRetriever

   Public Function GetItem(Of T, TComposition)( _
            ByVal predicate As Func(Of TComposition, Boolean)) _
            As T _
            Implements IPartRetriever.GetItem
      Dim retrieverForType = New PartRetrieverForType(Of T, TComposition)()
      If retrieverForType Is Nothing Then
         Throw New InvalidOperationException()
      End If
      Return retrieverForType.GetItem(predicate)
   End Function

End Class

Public Class PartRetrieverForType(Of T, TComposition)
   <ImportMany()> _
   Public _set As IEnumerable(Of Lazy(Of T, TComposition))

   Public Sub New()
      CompositionInitializer.SatisfyImports(Me)
   End Sub

#If DEBUG Then
   <ImportMany()> _
   Public _setTest As IEnumerable(Of Lazy(Of T, IDictionary(Of String, Object)))

   Private Function Report() As String
      Dim sb = New System.Text.StringBuilder()
      sb.AppendLine(GetType(TComposition).Name)
      For Each item In _setTest
         sb.Append(item.Value.[GetType]().Name & " : ")
         For Each pair In item.Metadata
            sb.Append(pair.Key & "(")
            If pair.Value Is Nothing Then
               sb.Append("<null>) : ")
            Else
               sb.Append(pair.Value.ToString() & ") : ")
            End If
         Next
         sb.AppendLine("")
      Next
      Return sb.ToString()
   End Function
#End If

   Public Function GetItem(ByVal predicate As System.Func(Of TComposition, Boolean)) _
               As T 
      Dim q = From x In _set _
          Where predicate.Invoke(x.Metadata) _
          Select x
      ' This assumes the first one is sufficient, perhaps raise an error if count > 1
      If q.Count() = 0 Then
         Return Nothing
      End If
      Return q.First().Value
   End Function
End Class

