﻿Option Infer On
Option Explicit On
Option Strict On

Imports System
Imports System.Linq
Imports System.ComponentModel
Imports System.ComponentModel.Composition
Imports KadGen.SLTestApp.Common
Imports System.Collections
Imports System.ComponentModel.DataAnnotations
Imports KadGen.SLTestApp.BusinessClasses

<ViewModelLookup(GetType(Customer))> _
<Export(GetType(IViewModelLookup(Of Customer)))> _
Public Class CustomerViewModelLookup
   Inherits ViewModelManyBase(Of Customer, CustomerViewModel)
   Implements IViewModelLookup(Of Customer)

   <Import()> _
   Public _list As CustomerViewModelMany
   Private _isFilled As Boolean

   Public Function LookupDisplay(ByVal id As Int32) _
               As String _
               Implements IViewModelLookup.Lookup
      If Not _isFilled Then
         _list.RetrieveMany(Nothing)
         _isFilled = True
      End If
      Dim customer = (From c In _list Where c.CustomerId = id).FirstOrDefault
      If customer IsNot Nothing Then
         Return customer.CustomerName
      End If
      Return "Customer ID not found: " & CStr(id)
   End Function
End Class

<ViewModelMany(GetType(Customer))> _
<Export(GetType(CustomerViewModelMany))> _
<PartCreationPolicy(CreationPolicy.NonShared)> _
Public Class CustomerViewModelMany
   Inherits ViewModelManyBase(Of Customer, CustomerViewModel)
End Class


<ViewModel(GetType(Customer))> _
<Export(GetType(CustomerViewModel))> _
<PartCreationPolicy(CreationPolicy.NonShared)> _
Public Class CustomerViewModel
   Inherits ViewModelBase(Of Customer)

   Public Property CustomerId() As Int32
      Get
         Return Model.CustomerId
      End Get
      Set(ByVal value As Int32)
         Model.CustomerId = value
      End Set
   End Property

   Public Property CustomerName() As String
      Get
         Return Model.CustomerName
      End Get
      Set(ByVal value As String)
         Model.CustomerName = value
      End Set
   End Property

End Class
