﻿Option Infer On
Option Explicit On
Option Strict On

Imports System
Imports System.Linq
Imports System.ComponentModel.Composition
Imports KadGen.SLTestApp.Common
Imports System.Collections
Imports System.ComponentModel.DataAnnotations
Imports KadGen.SLTestApp.BusinessClasses

<ViewModelMany(GetType(InvoiceLineItem))> _
<Export(GetType(InvoiceLineItemViewModelMany))> _
<PartCreationPolicy(CreationPolicy.NonShared)> _
Public Class InvoiceLineItemViewModelMany
   Inherits ViewModelManyBase(Of InvoiceLineItem, InvoiceLineItemViewModel)

End Class

<ViewModel(GetType(InvoiceLineItem))> _
<Export(GetType(InvoiceLineItemViewModel))> _
<PartCreationPolicy(CreationPolicy.NonShared)> _
Public Class InvoiceLineItemViewModel
   Inherits ViewModelBase(Of InvoiceLineItem)

   <Import()> _
   Public _products As ProductViewModelMany
   Private _areProductsFilled As Boolean

   Property InvoiceLineItemId() As Int32
      Get
         Return Model.InvoiceLineItemId
      End Get
      Set(ByVal value As Int32)
         Model.InvoiceLineItemId = value
      End Set
   End Property

   Property ProductId() As Int32
      Get
         Return Model.ProductId
      End Get
      Set(ByVal value As Int32)
         Model.ProductId = value
      End Set
   End Property

   ReadOnly Property ProductName() As String
      Get
         If Not _areProductsFilled Then
            _products.RetrieveMany(Nothing)
            _areProductsFilled = True
         End If
         Dim product = (From p In _products Where p.ProductId = ProductId).FirstOrDefault
         If product IsNot Nothing Then
            Return product.ProductName
         End If
         Return "Product ID not found: " & CStr(ProductId)
      End Get
   End Property

   Property Count() As Int32
      Get
         Return Model.Count
      End Get
      Set(ByVal value As Int32)
         Model.Count = value
      End Set
   End Property

   Property Cost() As Double
      Get
         Return Model.Cost
      End Get
      Set(ByVal value As Double)
         Model.Cost = value
      End Set
   End Property

   ReadOnly Property TotalCost() As Double
      Get
         Return Count * Cost
      End Get
   End Property

End Class
