﻿Option Infer On
Option Explicit On
Option Strict On

Imports System
Imports System.ComponentModel
Imports System.ComponentModel.Composition
Imports KadGen.SLTestApp.Common
Imports System.Collections
Imports System.ComponentModel.DataAnnotations
Imports KadGen.SLTestApp.BusinessClasses

<ViewModelMany(GetType(Invoice))> _
<Export(GetType(InvoiceViewModelMany))> _
<PartCreationPolicy(CreationPolicy.NonShared)> _
Public Class InvoiceViewModelMany
   Inherits ViewModelManyBase(Of Invoice, InvoiceViewModel)
End Class


<ViewModel(GetType(Invoice))> _
<Export(GetType(InvoiceViewModel))> _
<PartCreationPolicy(CreationPolicy.NonShared)> _
Public Class InvoiceViewModel
   Inherits ViewModelBase(Of Invoice)

   <Import()> _
   Public _customerLookup As IViewModelLookup(Of Customer)
   Private _areCustomersFilled As Boolean

   <Import()> _
   Public _cachedLineItems As InvoiceLineItemViewModelMany
   Private _areLineItemsFilled As Boolean

   <Required()> _
   <Editable(False)> _
   <Display(Description:="The primary key for the instance", Prompt:="ID")> _
   Public Property InvoiceId() As Int32
      Get
         Return Model.InvoiceId
      End Get
      Set(ByVal value As Int32)
         Model.InvoiceId = value
      End Set
   End Property

   Public Property InvoiceNumber() As String
      Get
         Return Model.InvoiceNumber
      End Get
      Set(ByVal value As String)
         Model.InvoiceNumber = value
      End Set
   End Property

   <Required()> _
   <Editable(False)> _
   <Display(Description:="The customer identifier", Prompt:="Customer")> _
   Public Property CustomerId() As Int32
      Get
         Return Model.CustomerId
      End Get
      Set(ByVal value As Int32)
         Model.CustomerId = value
      End Set
   End Property

   ReadOnly Property CustomerName() As String
      Get
         Return _customerLookup.Lookup(CustomerId)
      End Get
   End Property

   <Required()> _
   <Display(Description:="The invoice date", Prompt:="ID", AutoGenerateFilter:=True)> _
   Public Property InvoiceDate() As Date
      Get
         Return Model.InvoiceDate
      End Get
      Set(ByVal value As Date)
         Model.InvoiceDate = value
      End Set
   End Property

   Public Property MoreInfo() As String
      Get
         Return Model.MoreInfo
      End Get
      Set(ByVal value As String)
         Model.MoreInfo = value
      End Set
   End Property

   Public Property IsPaid() As Boolean
      Get
         Return Model.IsPaid
      End Get
      Set(ByVal value As Boolean)
         Model.IsPaid = value
      End Set
   End Property

   Public ReadOnly Property LineItems() As InvoiceLineItemViewModelMany
      Get
         If Not _areLineItemsFilled Then
            For Each item In Model.LineItems
               Dim vm = _cachedLineItems.CreateNew(item)
               _cachedLineItems.Add(vm)
            Next
            _areLineItemsFilled = True
         End If
         Return _cachedLineItems
      End Get
   End Property


End Class
